@@ -55,7 +55,11 @@ validateArgumentsInEnvironment <- function(envir, control, verbose, n.samples, s
   if (!missing(n.samples)) {
     tryCatch(n.samples <- as.integer(n.samples), warning = function(e)
              stop("'n.samples' argument to dbarts must be coerceable to integer type"))
-    if (n.samples < 0L) return("'n.samples' argument to dbarts must be a non-negative integer")
+    if (length(n.samples) != 1L) stop("'n.samples' must be of length 1")
+    if (is.null(n.samples))
+      stop("'n.samples' argument to dbarts cannot be NULL")
+    if (is.na(n.samples) || n.samples < 0L)
+      stop("'n.samples' argument to dbarts must be a non-negative integer")
     envir$control@n.samples <- n.samples
   } else if (controlIsMissing || is.na(control@n.samples)) {
     envir$control@n.samples <- formals(dbarts)[["n.samples"]]
@@ -64,7 +68,8 @@ validateArgumentsInEnvironment <- function(envir, control, verbose, n.samples, s
   if (!missing(sigma) && !is.na(sigma)) {
     tryCatch(sigma <- as.double(sigma), warning = function(e)
              stop("'sigma' argument to dbarts must be coerceable to numeric type"))
-    if (sigma <= 0.0) stop("'sigma' argument to dbarts must be positive")
+    if (length(sigma) != 1L) stop("'sigma' must be of length 1")
+    if (is.null(sigma) || sigma <= 0.0) stop("'sigma' argument to dbarts must be positive")
     
     envir$sigma <- sigma
   }
@@ -79,19 +84,22 @@ dbarts <- function(formula, data, test, subset, weights, offset, offset.test = o
 
   validateCall <- prepareCallWithArguments(matchedCall, quoteInNamespace(validateArgumentsInEnvironment), "control", "verbose", "n.samples", "sigma")
   validateCall <- addCallArgument(validateCall, 1L, sys.frame(sys.nframe()))
-  eval(validateCall, parent.frame(1L), asNamespace("dbarts"))
+  eval(validateCall, parent.frame(1L), getNamespace("dbarts"))
 
-  if (control@call[[1]] != call("NA")[[1]]) control@call <- matchedCall
+  if (length(control@call) == 1L && control@call == call("NA")) control@call <- matchedCall
   control@verbose <- verbose
 
-  parseDataCall <- prepareCallWithArguments(matchedCall, quoteInNamespace(parseData), "formula", "data", "test", "subset", "weights", "offset", "offset.test")
-  modelMatrices <- eval(parseDataCall, parent.frame(1L))
+  dataCall <- prepareCallWithArguments(matchedCall, quoteInNamespace(dbartsData), "formula", "data", "test", "subset", "weights", "offset", "offset.test")
+  data <- eval(dataCall, parent.frame(1L))
+  #cat("x address after dbartsData call: ", .Call("dbarts_getPointerAddress", data@x), "\n", sep = "")
   
-  data <- new("dbartsData", modelMatrices, attr(control, "n.cuts"), sigma)
+  data@n.cuts <- rep_len(attr(control, "n.cuts"), ncol(data@x))
+  data@sigma  <- sigma
   attr(control, "n.cuts") <- NULL
-  if (is.na(data@sigma))
-    data@sigma <- summary(lm(data@y ~ data@x, weights = data@weights))$sigma
-
+  
+  if (is.na(data@sigma) && !control@binary)
+    data@sigma <- summary(lm(data@y ~ data@x, weights = data@weights, offset = data@offset))$sigma
+    
   uniqueResponses <- unique(data@y)
   if (length(uniqueResponses) == 2 && all(sort(uniqueResponses) == c(0, 1))) control@binary <- TRUE
   ## bart will passthrough with offset == something no matter what, which we can NULL out
@@ -101,6 +109,7 @@ dbarts <- function(formula, data, test, subset, weights, offset, offset.test = o
   if (!control@binary && !is.null(data@offset.test) && all(data@offset.test == 0.0)) {
     data@offset.test <- NULL
   }
+  #cat("x address after updating data: ", .Call("dbarts_getPointerAddress", data@x), "\n", sep = "")
 
   parsePriorsCall <- prepareCallWithArguments(matchedCall, quoteInNamespace(parsePriors), "tree.prior", "node.prior", "resid.prior")
   parsePriorsCall <- setDefaultsFromFormals(parsePriorsCall, formals(dbarts), "tree.prior", "node.prior", "resid.prior")
@@ -111,7 +120,10 @@ dbarts <- function(formula, data, test, subset, weights, offset, offset.test = o
 
   model <- new("dbartsModel", priors$tree.prior, priors$node.prior, priors$resid.prior)
 
-  new("dbartsSampler", control, model, data)
+  result <- new("dbartsSampler", control, model, data)
+  #cat("x address after creating sampler: ", .Call("dbarts_getPointerAddress", data@x), "\n", sep = "")
+  #cat("x address in sampler$data: ", .Call("dbarts_getPointerAddress", result$data@x), "\n", sep = "")
+  result
 }
 
 setClassUnion("dbartsStateOrNULL", c("dbartsState", "NULL"))
@@ -132,12 +144,13 @@ dbartsSampler <-
                   if (!inherits(control, "dbartsControl")) stop("'control' must inherit from dbartsControl")
                   if (!inherits(model, "dbartsModel")) stop("'model' must inherit from dbartsModel")
                   if (!inherits(data, "dbartsData")) stop("'data' must inherit from dbartsData")
-
-                  control <<- control
-                  model   <<- model
-                  data    <<- data
-                  state   <<- NULL
-                  pointer <<- .Call("dbarts_create", control, model, data)
+                  
+                  .self$control <- control
+                  .self$model   <- model
+                  .self$data    <- data
+                  
+                  .self$state   <- NULL
+                  .self$pointer <- .Call(C_dbarts_create, .self$control, .self$model, .self$data)
                   
                   callSuper(...)
                 },
@@ -148,7 +161,7 @@ dbartsSampler <-
                   if (missing(numSamples)) numSamples <- NA_integer_
                   
                   ptr <- getPointer()
-                  samples <- .Call("dbarts_run", ptr, as.integer(numBurnIn), as.integer(numSamples))
+                  samples <- .Call(C_dbarts_run, ptr, as.integer(numBurnIn), as.integer(numSamples))
 
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
                     storeState(ptr)
@@ -157,8 +170,8 @@ dbartsSampler <-
                   
                   samples
                 },
-                copy = function(shallow = FALSE)
-                {
+                copy = function(shallow = FALSE) {
+                  'Creates a deep or shallow copy of the sampler.'
                   dupe <-
                     if (shallow) {
                       dbartsSampler$new(control, model, data)
@@ -166,29 +179,29 @@ dbartsSampler <-
                       newData <- data
                       ## only need to dupe things that can be changed internally, as the rest will be
                       ## simply swapped out
-                      newData@x <- .Call("dbarts_deepCopy", data@x)
+                      newData@x <- .Call(C_dbarts_deepCopy, data@x)
                       if (!is.null(data@x.test)) {
-                        newData@x.test <- .Call("dbarts_deepCopy", data@x.test)
+                        newData@x.test <- .Call(C_dbarts_deepCopy, data@x.test)
                       }
                       dupe <- dbartsSampler$new(control, model, newData)
                     }
 
                   if (!is.null(state)) {
                     newState <- state
-                    newState@fit.tree <- .Call("dbarts_deepCopy", state@fit.tree)
-                    newState@fit.total <- .Call("dbarts_deepCopy", state@fit.total)
+                    newState@fit.tree <- .Call(C_dbarts_deepCopy, state@fit.tree)
+                    newState@fit.total <- .Call(C_dbarts_deepCopy, state@fit.total)
                     if (!is.null(data@x.test)) {
-                      newState@fit.test <- .Call("dbarts_deepCopy", state@fit.test)
+                      newState@fit.test <- .Call(C_dbarts_deepCopy, state@fit.test)
                     }
-                    newState@sigma <- .Call("dbarts_deepCopy", state@sigma)
-                    newState@runningTime <- .Call("dbarts_deepCopy", state@runningTime)
-                    newState@trees <- .Call("dbarts_deepCopy", state@trees)
+                    newState@sigma <- .Call(C_dbarts_deepCopy, state@sigma)
+                    newState@runningTime <- .Call(C_dbarts_deepCopy, state@runningTime)
+                    newState@trees <- .Call(C_dbarts_deepCopy, state@trees)
 
                     dupe$setState(newState)
                   }
                   
                   dupe
-                },    
+                },
                 show = function() {
                   'Pretty prints the object.'
                   
@@ -217,27 +230,62 @@ dbartsSampler <-
                     
                   ##writeLines(xDisplay)
                   
+                  invisible(NULL)
                 },
                 setControl = function(newControl) {
                   'Sets the control object for the sampler to a new one. Preserves the call() slot.'
                   
-                  if (!inherits(newControl, "dbartsControl")) stop("'control' must inherit from dbartsControl.")
+                  if (!inherits(newControl, "dbartsControl")) stop("'control' must inherit from dbartsControl")
 
                   newControl@binary <- control@binary
                   newControl@call   <- control@call
                   
                   ptr <- getPointer()
-                  control <<- newControl
-                  .Call("dbarts_setControl", ptr, control)
+                  selfEnv <- parent.env(environment())
+                  
+                  selfEnv$control <- newControl
+                  .Call(C_dbarts_setControl, ptr, control)
+                  
+                  invisible(NULL)
+                },
+                setModel = function(newModel) {
+                  'Sets the model object for the sampler to a new one.'
+                  
+                  if (!inherits(newModel, "dbartsModel")) stop("'model' must inherit from dbartsModel")
+                  ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
+                  
+                  selfEnv$model <- newModel
+                  .Call(C_dbarts_setModel, ptr, model)
+                  
+                  invisible(NULL)
+                },
+                setData = function(newData, updateState = NA) {
+                  'Sets the data object for the sampler to a new one. Preserves the n.cuts and sigma slots.'
+                  
+                  if (!inherits(newData, "dbartsData")) stop("'data' must inherit from dbartsData")
+                  
+                  newData@n.cuts <- data@n.cuts
+                  newData@sigma  <- data@sigma
+                  
+                  ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
+                  
+                  selfEnv$data <- newData
+                  .Call(C_dbarts_setData, ptr, data)
+                  
+                  if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
+                    storeState(ptr)
                   
                   invisible(NULL)
                 },
                 setResponse = function(y, updateState = NA) {
                   'Changes the response against which the sampler is fitted.'
                   ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
                   
-                  data@y <<- as.double(y)
-                  .Call("dbarts_setResponse", ptr, data@y)
+                  selfEnv$data@y <- as.double(y)
+                  .Call(C_dbarts_setResponse, ptr, data@y)
 
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
                     storeState(ptr)
@@ -247,7 +295,8 @@ dbartsSampler <-
                 setOffset = function(offset, updateState = NA) {
                   'Changes the offset slot used to adjust the response.'
                   ptr <- getPointer()
-
+                  selfEnv <- parent.env(environment())
+                  
                   offset.test <- NA
                   if (is.null(offset)) {
                     if (identical(data@testUsesRegularOffset, TRUE)) offset.test <- NULL
@@ -276,11 +325,11 @@ dbartsSampler <-
                     }
                   }
 
-                  data@offset <<- offset
-                  .Call("dbarts_setOffset", ptr, data@offset)
+                  selfEnv$data@offset <- offset
+                  .Call(C_dbarts_setOffset, ptr, data@offset)
                   if (!identical(offset.test, NA)) {
-                    data@offset.test <<- offset.test
-                    .Call("dbarts_setTestOffset", ptr, data@offset.test)
+                    selfEnv$data@offset.test <- offset.test
+                    .Call(C_dbarts_setTestOffset, ptr, data@offset.test)
                   }
 
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
@@ -303,18 +352,20 @@ dbartsSampler <-
                   x <- if (is.matrix(x)) matrix(as.double(x), nrow(x)) else as.double(x)
                   
                   ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
+                  
                   updateSuccessful <-
                     if (columnIsMissing) {
-                      data@x <<- x
-                      .Call("dbarts_setPredictor", ptr, x)
+                      selfEnv$data@x <- x
+                      .Call(C_dbarts_setPredictor, ptr, data@x)
                     } else {
-                      .Call("dbarts_updatePredictor", ptr, x, as.integer(column))
+                      .Call(C_dbarts_updatePredictor, ptr, x, as.integer(column))
                     }
                   
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
                     storeState(ptr)
 
-                  return(updateSuccessful)
+                  updateSuccessful
                 },
                 setTestPredictor = function(x.test, column, updateState = NA) {
                   'Changes a single column of the test predictor matrix.'
@@ -329,12 +380,14 @@ dbartsSampler <-
                   }
                   
                   ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
+                  
                   if (columnIsMissing) {
-                    data@x.test <<- validateXTest(x.test, ncol(data@x), colnames(data@x))
-                    .Call("dbarts_setTestPredictor", ptr, data@x.test)
+                    selfEnv$data@x.test <- validateXTest(x.test, attr(data@x, "term.labels"), ncol(data@x), colnames(data@x), attr(data@x, "drop"))
+                    .Call(C_dbarts_setTestPredictor, ptr, data@x.test)
                   } else {
                     x.test <- if (is.matrix(x.test)) matrix(as.double(x.test), nrow(x.test)) else as.double(x.test)
-                    .Call("dbarts_updateTestPredictor", ptr, x.test, as.integer(column))
+                    .Call(C_dbarts_updateTestPredictor, ptr, x.test, as.integer(column))
                   }
                   
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
@@ -345,8 +398,9 @@ dbartsSampler <-
                 setTestPredictorAndOffset = function(x.test, offset.test, updateState = NA) {
                    'Changes the test predictor matrix, and optionally the test offset.'
                    ptr <- getPointer()
+                   selfEnv <- parent.env(environment())
 
-                   x.test <- validateXTest(x.test, ncol(data@x), colnames(data@x))
+                   x.test <- validateXTest(x.test, attr(data@x, "term.labels"), ncol(data@x), colnames(data@x), attr(data@x, "drop"))
 
                    if (!missing(offset.test)) {
                      if (is.null(x.test)) {
@@ -362,14 +416,14 @@ dbartsSampler <-
                          }
                        }
                      }
-                     data@testUsesRegularOffset <<- FALSE
+                     selfEnv$data@testUsesRegularOffset <- FALSE
 
-                     data@x.test <<- x.test
-                     data@offset.test <<- offset.test
-                     .Call("dbarts_setTestPredictorAndOffset", ptr, data@x.test, data@offset.test)
+                     selfEnv$data@x.test <- x.test
+                     selfEnv$data@offset.test <- offset.test
+                     .Call(C_dbarts_setTestPredictorAndOffset, ptr, data@x.test, data@offset.test)
                    } else {
-                     data@x.test <<- x.test
-                     .Call("dbarts_setTestPredictorAndOffset", ptr, data@x.test, NA_real_)
+                     selfEnv$data@x.test <- x.test
+                     .Call(C_dbarts_setTestPredictorAndOffset, ptr, data@x.test, NA_real_)
                    }
 
                    if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
@@ -380,16 +434,17 @@ dbartsSampler <-
                 setTestOffset = function(offset.test, updateState = NA) {
                   'Changes the test offset.'
                   ptr <- getPointer()
+                  selfEnv <- parent.env(environment())
 
-                  data@testUsesRegularOffset <<- FALSE
+                  selfEnv$data@testUsesRegularOffset <- FALSE
                   if (!is.null(offset.test)) {
                     if (is.null(data@x.test)) stop("when test matrix is NULL, test offset must be as well")
                     if (length(offset.test) == 1) offset.test <- rep_len(offset.test, nrow(data@x.test))
                     if (length(offset.test) != nrow(data@x.test))
                       stop("length of test offset must be equal to number of rows in test matrix")
                   }
-                  data@offset.test <<- offset.test
-                  .Call("dbarts_setTestOffset", ptr, data@offset.test)
+                  selfEnv$data@offset.test <- offset.test
+                  .Call(C_dbarts_setTestOffset, ptr, data@offset.test)
                   
                   if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
                     storeState(ptr)
@@ -398,16 +453,18 @@ dbartsSampler <-
                 },
                 getPointer = function() {
                   'Returns the underlying reference pointer, checking for consistency first.'
-                  if (.Call("dbarts_isValidPointer", pointer) == FALSE) {
+                  selfEnv <- parent.env(environment())
+                  
+                  if (.Call(C_dbarts_isValidPointer, pointer) == FALSE) {
                     oldVerbose <- control@verbose
-                    control@verbose <<- FALSE
-                    pointer <<- .Call("dbarts_create", control, model, data)
+                    selfEnv$control@verbose <- FALSE
+                    selfEnv$pointer <- .Call(C_dbarts_create, control, model, data)
                     if (!is.null(state)) {
-                      state <<- .Call("dbarts_restoreState", pointer, state)
+                      selfEnv$state <- .Call(C_dbarts_restoreState, pointer, state)
                     }
                     if (control@verbose != oldVerbose) {
-                      control@verbose <<- oldVerbose
-                      .Call("dbarts_setControl", pointer, control)
+                      selfEnv$control@verbose <- oldVerbose
+                      .Call(C_dbarts_setControl, pointer, control)
                     }
                   }
                   
@@ -417,29 +474,116 @@ dbartsSampler <-
                   'Sets the internal state from a cache.'
                   if (!is(newState, "dbartsState")) stop("'state' must inherit from dbartsState")
                   
-                  if (.Call("dbarts_isValidPointer", pointer) == FALSE) {
+                  selfEnv <- parent.env(environment())
+                  if (.Call(C_dbarts_isValidPointer, pointer) == FALSE) {
                     oldVerbose <- control@verbose
-                    control@verbose <<- FALSE
-                    pointer <<- .Call("dbarts_create", control, model, data)
+                    selfEnv$control@verbose <- FALSE
+                    selfEnv$pointer <- .Call(C_dbarts_create, control, model, data)
                     if (control@verbose != oldVerbose) {
-                      control@verbose <<- oldVerbose
-                      .Call("dbarts_setControl", pointer, control)
+                      selfEnv$control@verbose <- oldVerbose
+                      .Call(C_dbarts_setControl, pointer, control)
                     }
                   }
                   if (!is.null(newState)) {
-                    state <<- .Call("dbarts_restoreState", pointer, newState)
+                    selfEnv$state <- .Call(C_dbarts_restoreState, pointer, newState)
                   }
 
                   invisible(NULL)
                 },
                 storeState = function(ptr = getPointer()) {
                   'Updates the cached internal state used for saving/loading.'
+                  selfEnv <- parent.env(environment())
                   if (is.null(state)) {
-                    state <<- .Call("dbarts_createState", ptr)
+                    selfEnv$state <- .Call(C_dbarts_createState, ptr)
                   } else {
-                    .Call("dbarts_storeState", ptr, state)
+                    .Call(C_dbarts_storeState, ptr, state)
                   }
 
                   invisible(NULL)
-                })
-              )
+                },
+                printTrees = function(treeNums = seq_len(control@n.trees)) {
+                  'Produces an info dump of the internal state of the trees.'
+                  
+                  ptr <- getPointer()
+                  invisible(.Call(C_dbarts_printTrees, ptr, as.integer(treeNums)))
+                },
+                plotTree = function(treeNum, treePlotPars = list(nodeHeight = 12, nodeWidth = 40, nodeGap = 8), ...) {
+                  'Minimialist visualization of tree branching and contents.'
+                  
+                  cutPoints <- createCutPoints(.self)
+  
+                  tree <- buildTree(strsplit(gsub("\\.", "\\. ", state@trees[treeNum]), " ", fixed = TRUE)[[1]])
+                  tree$remainder <- NULL
+                  
+                  tree$indices <- seq_len(nrow(data@x))
+                  tree <- fillObservationsForNode(tree, .self, cutPoints)
+                  
+                  tree <- fillPlotInfoForNode(tree, .self, state@fit.tree[,treeNum])
+                  
+                  maxDepth <- getMaxDepth(tree)
+                  
+                  tree <- fillPlotCoordinatesForNode(tree, maxDepth, 1L, 1L)
+                  numEndNodes <- tree$index - 1L
+                  tree$index <- NULL
+                  
+                  plotHeight <- treePlotPars$nodeHeight * maxDepth + treePlotPars$nodeGap * (maxDepth - 1)
+                  dotsList <- list(...)
+                  dotsList$mar = c(0, 0, 0, 0)
+                  par(dotsList)
+                  plot(NULL, type = "n", bty = "n", xaxt = "n", yaxt = "n", xlab = "", ylab = "",
+                       xlim = c(0, treePlotPars$nodeWidth * numEndNodes), ylim = c(0, plotHeight))
+                  plotNode(tree, .self, cutPoints, treePlotPars)
+                  
+                  invisible(NULL)
+                }
+                #crossvalidate = function(K = 5L, n.reps = 200L, n.burn = c(control@n.burn, control@n.burn %/% 5L),
+                #                         loss = if (control@binary) "mcr" else "rmse", n.threads = guessNumCores(),
+                #                         n.trees, k, power, base, drop = TRUE) {
+                #  'K-fold crossvalidates across sets of parameters.'
+                #  
+                #  if (missing(n.trees)) n.trees <- control@n.trees
+                #  if (missing(k))       k <- model@node.prior@k
+                #  if (missing(power))   power <- model@tree.prior@power
+                #  if (missing(base))    base <- model@tree.prior@base
+                #  
+                #  if (is.function(loss)) loss <- list(loss, parent.frame(1L))
+                #  
+                #  kOrder <- order(k, decreasing = TRUE)
+                #  kOrder.inv <- kOrder; kOrder.inv[kOrder] <- seq_along(kOrder)
+                #  k <- k[kOrder] 
+                #  
+                #  n.trees <- as.integer(n.trees)
+                #  k       <- as.double(k)
+                #  power   <- as.double(power)
+                #  base    <- as.double(base)
+                #  drop    <- as.logical(drop)
+                #  
+                #  ptr <- getPointer()
+                #  result <- .Call(C_dbarts_xbart, ptr, as.integer(K), as.integer(n.reps), as.integer(n.burn), loss, as.integer(n.threads),
+                #                  n.trees, k, power, base, drop)
+                #  
+                #  if (is.null(result) || is.null(dim(result))) return(result)
+                #  
+                #  varNames <- c("n.trees", "k", "power", "base")
+                #  if (identical(drop, TRUE))
+                #    varNames <- varNames[sapply(varNames, function(varName) if (length(get(varName)) > 1L) TRUE else FALSE)]
+                #  
+                #  if ("k" %in% varNames && any(kOrder != seq_along(k))) {
+                #    indices <- rep(list(bquote()), length(dim(result)))
+                #    indices[[1L + which(varNames == "k")]] <- kOrder.inv
+                #    indexCall <- as.call(c(list(as.name("["), quote(result)), indices))
+                #    result <- eval(indexCall)
+                #    k <- k[kOrder.inv]
+                #  }
+                #  
+                #  dimNames <- vector("list", length(dim(result)))
+                #  for (i in seq_along(varNames)) {
+                #    x <- get(varNames[i])
+                #    dimNames[[i + 1L]] <- as.character(if (is.double(x)) signif(x, 2L) else x)
+                #  }
+                #  names(dimNames) <- c("rep", varNames)
+                #  dimnames(result) <- dimNames
+                #  
+                #  result
+                #}
+              ))
